
/*
**   ##
**    #                 #
**    #
**    #   ####   ### # ##  # ###
**    #  #    # #   ##  #  ##   #
**    #  #    # #    #  #  #    #
**    #  #    # #    #  #  #    #
**    #  #    # #   ##  #  #    #
**   ###  ####   ### # ### #    #
**                   #
**              #    #
**               ####
*/

/*
**
**   login for LUROX ABC1600
**
**   Usage:
**
**      login [ -p ] [ name ]
**
**   Author:
**       _
**      Goran Larsson,  hacker@isadora.ikp.liu.se
**                             _
**	Copyright (C) 1988 by Goran Larsson
**
*/

static char HoHid[] = "@(#) login, (C) 1988 Hacker of Hackefors";
static char RCSid[] = "$Header: /home2/temp/usr.src-from-isadora/bin/RCS/login.c,v 1.4 1989/12/20 00:11:32 root Exp $";

/*
 *	$Log: login.c,v $
 * Revision 1.4  1989/12/20  00:11:32  root
 * Env. var. TZ set from the file /etc/TZ
 *
 * Revision 1.3  88/12/16  09:40:09  root
 * Small change in test if /etc/nologin existed, caused login to
 * core for users not root.
 * 
 * Revision 1.2  88/12/16  00:25:00  root
 * Did not clear name when password wrong.
 * 
 * Revision 1.1  88/12/15  22:37:34  root
 * Initial revision
 * 
 */


#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <time.h>
#include <termio.h>
#include <fcntl.h>
#include <utmp.h>
#include <signal.h>
#include <pwd.h>		/* modified */
#include <stdio.h>
#include <lastlog.h>		/* new */
#include <errno.h>
#include <grp.h>
#include <dnix/fcodes.h>
#include <dnix/log.h>
#include <string.h>

#define FALSE (0)
#define TRUE (-1)

#define INTRUDER 5	/* assume intruder after this many login fails     */
#define DEFEND 120	/* defend against intruder by sleeping this many s */
#define TTYPROT 0622	/* tty protection, (- rw- -w- -w-) when logged in  */
#define	FAILMAX	20	/* Number of tries before account is closed */

#define L_OK   0	/* normal termination code		*/
#define L_BAD  1	/* unsuccesful termination code		*/
#define L_DISC 2	/* as above but with "broken" message	*/

#define TZMAX 16	/* max number of chars in the TZ variable */

extern char *strncpy ();
extern char *strcpy ();
extern char *strcat ();
extern char *strrchr ();
extern char *malloc ();
extern char *realloc ();
extern char *getenv ();
extern char *ttyname ();
extern char *crypt ();
extern char *getpass ();

extern char **environ;

int timedout ();

/*
** items read from gettydef file
*/

struct {
	int timeout;
	char herald[80];
	char login0[32];
	char login1[32];
	char login2[32];
	char passwd[32];
	char triggr[8];
	char strategy;
	int bauds[4];
	struct termio tty;
} ttydef;

/*
** files
*/

char Fpassd[]	   = "/etc/passwd" ; 		/* Password file */
char Fgettydef[]   = "/etc/gettydef";		/* definitions */
char Fnologin[]    = "/etc/nologin";		/* machine lock file */
char Fhushlogin[]  = ".hushlogin";		/* be quiet */
char Flastlog[]    = "/usr/adm/lastlog";	/* security file */
char Fsh[]         = "/bin/sh";			/* old fellow */
char Futmp[]       = "/etc/utmp";		/* who's on */
char Fwtmp[]       = "/usr/adm/wtmp";		/* who's been on */
char Fmotd[]       = "/etc/motd";		/* news from root */
char Fdash[1+14+1] = "-";			/* used when forking shell */
char Ftz[]         = "/etc/TZ";			/* file with timezone info */

/*
** directories
*/

char Dmail[16+14]  = "/usr/spool/mail/";	/* post office */
char Dpath[]       = "/usr/local/bin:/bin:/usr/bin:";

/*
** default prompts et. al.
*/

char defheral[] = "\r\n";
char deflogin[] = "Login: ";
char defpassw[] = "Password: ";

/*
** messages
*/

char msgdenied[]    = "Login denied\r\n";
char msgincorrect[] = "Login incorrect\r\n";
char msgnodir[]     = "No directory\r\n";
char msglastlog[]   = "Last login: %.*s on %.*s\r\n";
char msgfailures[]  = "\007%d failures since last login\007\r\n";
char msgmail[]      = "You have %smail.\r\n";	/* %s -> "new " or "" */
char msgnosh[]      = "No shell\r\n";
char msgdisco[]     = "\r\nDisconnected\r\n";
char msgtime[]      = "\r\nTimeout period expired\r\n";
char msgmalloc[]    = "malloc: no memory\r\n";
char msgintruder[]  = "INTRUDER ALERT on %s, \"%s\", \"%s\"\n";

/*
** structures and variables
*/

struct passwd nouser = {
	"",		/* login name				*/
	"Gazonk",	/* encrypted password			*/
	-1,		/* numerical user ID			*/
	-1,		/* numerical group ID			*/
	-1,		/* max user disk usage (not used)	*/
	"",		/* data for password aging (not used)	*/
	"No Virus Here",/* comment				*/
	"",		/* gecos field (not used)		*/
	"",		/* initial working directory		*/
	""		/* program to use as shell		*/
};

struct utmp utmp;
struct passwd *pwd;
#define NAMELEN (sizeof utmp.ut_name)
struct termio oldtty;
char *envinit[] = { 0 };
int getty;
char tzstring[TZMAX+1];
FILE *file;


/*
**		l o g i n
*/

main ( argc, argv )
int argc;
char *argv[];

{
	register char *namep;
	enum { NOTPRESENT, PRESENT } pflag = NOTPRESENT;
	long ofs;
	int fd;
	int ix;
	int c;
	int tries;
	int invalid;
	int quietlog;
	int fails ;
	char *ttyn;
	char *tty;
	char **envnew;
	char *pp;

	/*
	** remember if invoked from getty
	*/

	getty = FALSE;
	if ( argv[0][0] == '-' ) {
		getty = TRUE;
	}

	/*
	** read system parameters from file
	*/

	if ( ( fd = open ( Fgettydef, O_RDONLY ) ) < 0 ) {

		/* file not found, use default values */

		ttydef.timeout = 60;
		(void) strcpy ( ttydef.login0, deflogin );
		(void) strcpy ( ttydef.login2, deflogin );
		(void) strcpy ( ttydef.passwd, defpassw );
	} else {

		/* read defs from file */

		/* first read network record */

		lseek ( fd, 0L, 0 );
		read ( fd, (char *)&ttydef, sizeof ttydef );

		/* now see if trigger fits */

		if ( strncmp (
		  ttyname ( 0 ),
		  ttydef.triggr,
		  strlen ( ttydef.triggr )
		) != 0 ) {

			/* trigger failed, read normal tty record */
			/* NOTE: ttyslot() are zero based in abcenix */

			ofs = ( ttyslot () + 1 ) * sizeof ttydef;
			lseek ( fd, ofs, 0 );
			read ( fd, (char *)&ttydef, sizeof ttydef );
		}

		close ( fd );
	}

	/*
	** get terminal parameters (do not use database values)
	*/

	ioctl ( 0, TCGETA, &ttydef.tty );

	/*
	** setup a time limit for login
	*/

	signal ( SIGALRM, timedout );
	alarm ( ttydef.timeout );

	/*
	** ignore these signals during login procedure
	*/

	signal ( SIGQUIT, SIG_IGN );
	signal ( SIGINT, SIG_IGN );

	/*
	** -p is used to tell login not to destroy the environment
	*/

	while ( argc > 1 ) {
		if (strcmp ( argv[1], "-p" ) == 0 ) {
			argc--;
			argv++;
			pflag = PRESENT;
			continue;
		}
		break;
	}

	/*
	** close unused file descriptors
	*/
	
	for ( ix = 3; ix < _NFILE; ix++ ) {
		close ( ix );
	}

	ttyn = ttyname ( 0 );
	if ( ( ttyn == (char *)0 ) || ( *ttyn == '\0' ) ) {
		ttyn = "/dev/tty??";
	}
	tty = strrchr ( ttyn, '/' ) + 1;

	/*
	** start the login procedure
	*/

	tries = 0;
	invalid = FALSE;
	do {
		/*
		** grab username; we get it from two sources:
		**   1) on the command line as argument to login
		**   2) the user types it in
		*/

		*utmp.ut_name = '\0';

		/*
		** 1) on command line
		*/

		if ( argc > 1 ) {
			strcpy ( utmp.ut_name, argv[1] );
			argc = 0;			/* one shot */
		}

		/*
		** 2) ask user now
		*/

		if ( utmp.ut_name[0] == '\0' ) {
			savetty ();
			rawtty ();
			while ( utmp.ut_name[0] == '\0' ) {
				if ( !getname (
				  utmp.ut_name, NAMELEN, ttydef.timeout
				) ) {
					leave ( L_DISC );  /* BREAK detected */
				}
			}
			restoretty ();
		}

		/*
		** check if user exist
		*/

login:
		if ( ( pwd = getpwnam ( utmp.ut_name ) ) == NULL ) {
			pwd = &nouser;
		}
		(void) endpwent ();
		/*
		** if users shell is '*' then change root and use new tree
		*/

		if ( *pwd->pw_shell == '*' ) {
			if ( chroot ( pwd->pw_dir ) ) {
				printf ( msgdenied );
				fflush ( stdout );
				leave ( L_BAD );
			}
			goto login;		/* restart */
		}

		/*
		** if required: ask for password and check correctness
		*/

		invalid = FALSE;
		flushtty ();
		if ( *pwd->pw_passwd ) {
			pp = getpass ( ttydef.passwd );
			namep = crypt ( pp, pwd->pw_passwd );
			if ( strcmp ( namep, pwd->pw_passwd ) ) {
				invalid = TRUE;
			} else {
				(void) strcpy ( pp, "KateBush" ); /* security */
			}
		}

		/*
		** tell user that he did not make it (if he didn't)
		*/

		if ( invalid ) {
			printf ( msgincorrect );

			/*
			** Count the failure
			*/

			if ( ( pwd != &nouser )
			  && ( ( fd = open ( Flastlog, O_RDWR ) ) >= 0 ) ) {
				static struct lastlog ll = { 0, "", 0, 0 };
				lseek ( fd, (long)pwd->pw_uid * sizeof (struct lastlog), 0 );
				read ( fd, (char *)&ll, sizeof ll );
				lseek ( fd, (long)pwd->pw_uid * sizeof(struct lastlog), 0 );
				fails = ll.ll_failures++;
				ll.ll_totfail++;
				write ( fd, (char *)&ll, sizeof ll );
				close ( fd );
			}
			if (fails > FAILMAX) { closeacc(utmp.ut_name) ; }
			if ( ++tries >= INTRUDER ) {

				/*
				** Issue warning to console and logfile
				*/

				dnix (
				  F_LOGC, E_WARNING,
				  msgintruder,
				  ttyn, utmp.ut_name, pp
				);

				/*
				** Hang up the line
				*/

				ioctl ( 0, TCGETA, &ttydef.tty );
				ttydef.tty.c_cflag |= HUPCL;
				ioctl ( 0, TCSETA, &ttydef.tty );
				close ( 0 ), close ( 1 ), close ( 2 );
				sleep ( DEFEND );
				leave ( L_BAD );
			}
			continue;	/* but not with the code below! */
		}

		/*
		** If user not super-user, check for logins disabled.
		*/

		if ( pwd->pw_uid != 0 ) {
			file = fopen ( Fnologin, "r" );
			if ( file > 0 ) {
				while ( ( c = getc ( file ) ) != EOF ) {
					putchar ( c );
				}
				fflush ( stdout );
				sleep ( 5 );
				leave ( L_OK );
			}
		}
/* *#*#*#*#*#* */
		if (checkname(utmp.ut_name,2) == -1) {}
		/*
		** use /bin/sh if nothing else specified
		*/

		if ( *pwd->pw_shell == '\0' ) {
			pwd->pw_shell = Fsh;
		}

		/*
		** change to users directory
		*/

		if ( chdir ( pwd->pw_dir ) < 0 ) {
			printf ( "%s%s", msgnodir, msgdenied );
			fflush ( stdout );
			leave ( L_BAD );
		}

	} while ( invalid );


	/*
	** must login now, turn off timeout
	*/

	alarm ( 0 );

	/*
	** make entry in utmp file
	*/

	time ( &utmp.ut_time );
	strcpy ( utmp.ut_line, tty );
	/* NOTE: ttyslot() are zero based in abcenix */
	ofs = ttyslot ();
	if ( ( ofs >= 0 ) && ( fd = open ( Futmp, O_WRONLY ) ) >= 0 ) {
		lseek ( fd, ofs * sizeof utmp, 0 );
		write ( fd, (char *)&utmp, sizeof utmp );
		close ( fd );
	}

	/*
	** make entry in wtmp file
	*/

	if ( ( fd = open ( Fwtmp, O_WRONLY|O_APPEND ) ) >= 0 ) {
		write ( fd, (char *)&utmp, sizeof utmp );
		close ( fd );
	}

	/*
	** check if we should login quietly (i.e. uucp)
	*/

	quietlog = access ( Fhushlogin, F_OK ) == 0;

	/*
	** display info about last login
	*/

	if ( ( fd = open ( Flastlog, O_RDWR ) ) >= 0 ) {
		struct lastlog ll;

		lseek ( fd, (long)pwd->pw_uid * sizeof (struct lastlog), 0 );
		if ( ( read ( fd, (char *)&ll, sizeof ll ) == sizeof ll )
		  && ( ll.ll_time != 0 )
		  && ( !quietlog ) ) {
			printf (
			  msglastlog,
			  24-5, (char *)ctime ( &ll.ll_time ),
			  sizeof ll.ll_line, ll.ll_line
			);
			if ( ll.ll_failures ) {
				printf ( msgfailures, ll.ll_failures );
			}
		}

		/*
		** make new entry in lastlog file
		*/

		lseek ( fd, (long)pwd->pw_uid * sizeof (struct lastlog), 0 );
		time ( &ll.ll_time );
		strcpy ( ll.ll_line, tty );
		ll.ll_failures = (short)0;
		write ( fd, (char *) &ll, sizeof ll );
		close ( fd );
	}

	/*
	** change owner and protection on terminal device
	*/

	chown ( ttyn, pwd->pw_uid, tty_gid ( pwd->pw_gid ) );
	chmod ( ttyn, TTYPROT );

	/*
	** reset priority
	*/

	nice ( -40 );			/* must be su for this */
	nice ( 20 );
	nice ( 0 );

	/*
	** read timezone info
	*/

	ix = 0;
	file = fopen ( Ftz, "r" );
	if ( file > 0 ) {
		while ( ( c = getc ( file ) ) != EOF ) {
			if ( ( ix >= TZMAX ) || ( c < ' ' ) ) {
				break;
			}
			tzstring[ix++] = c;
			tzstring[ix] = '\0';
		}
		(void) fclose ( file );
	} else {
		*tzstring = '\0';
	}

	/*
	** set group id and user id
	*/

	setgid ( pwd->pw_gid );		/* must be before setuid */
	setuid ( pwd->pw_uid );		/* privileges lost now   */

	/*
	** destroy environment unless user has asked to preserve it
	*/

	if ( pflag == NOTPRESENT ) {
		environ = envinit;
	}

	/*
	** set up environment, this time without destruction.
	** copy the environment before setenv'ing
	*/

	ix = 0;
	while (environ[ix] != NULL) {
		ix++;
	}
	envnew = (char **) malloc ( sizeof (char *) * (ix + 1) );
	for (; ix >= 0; ix-- ) {
		envnew[ix] = environ[ix];
	}
	environ = envnew;

	/*
	** make some usefull entries in the environment
	*/

	setenv ( "HOME=", pwd->pw_dir, 1 );
	setenv ( "SHELL=", pwd->pw_shell, 1 );
	setenv ( "USER=", pwd->pw_name, 1 );
	setenv ( "PATH=", Dpath, 0 );
	strcat ( Dmail, pwd->pw_name );
	setenv ( "MAIL=", Dmail, 1 );
	setenv ( "TZ=", tzstring, 1 );

	/*
	** construct argv[0] for the user shell
	*/

	if ( ( namep = strrchr ( pwd->pw_shell, '/') ) == NULL ) {
		namep = pwd->pw_shell;
	} else {
		namep++;
	}
	strcat ( Fdash, namep );	/* prepend dash, i.e. -csh */

	/*
	** if root is logging in, do something usefull (someday)
	*/

	if ( pwd->pw_uid == 0 ) {
		/*VOID*/
	}

	if ( !quietlog ) {
		struct stat st;

		/*
		** display the Message Of The Day file
		*/

		showmotd ();

		/*
		** check for new mail
		*/

		if ( stat ( Dmail, &st ) == 0 && st.st_size != 0 ) {
			printf (
			  msgmail,
			  ( st.st_mtime > st.st_atime ) ? "new " : ""
			);
		}
	}

	/*
	** restore signals to their default actions
	*/

	signal ( SIGALRM, SIG_DFL );
	signal ( SIGQUIT, SIG_DFL );
	signal ( SIGINT, SIG_DFL );

	/*
	** here we go!
	*/

	execlp ( pwd->pw_shell, Fdash, 0 );

	/*
	** that did not last long!
	*/

	printf ( msgnosh );
	leave ( L_BAD );
}


/*
** restore original terminal setting and exit
*/

leave ( val )
int val;

{
	switch ( val ) {
	case L_OK :
		break;
	case L_BAD :
		break;
	case L_DISC :
		printf ( msgdisco );
		break;
	}
	ioctl ( 0, TCSETAF, &oldtty );	/* restore old setting */
	exit ( val );
	/*NOTREACHED*/
}

/*
** set terminal in RAW mode
*/

rawtty ()

{
	struct termio tty;

	ioctl ( 0, TCGETA, &tty );
	tty.c_iflag = IGNPAR | ISTRIP;
	tty.c_oflag = 0;
/*	tty.c_cflag			we keep this beast untouched */
	tty.c_lflag = 0;
	tty.c_line = 0;
	tty.c_cc[VMIN] = 1;
	tty.c_cc[VTIME] = 0;
	ioctl ( 0, TCSETAF, &tty );
}

/*
** save and restore terminal mode
*/

struct termio savtty;

savetty ()

{
	ioctl ( 0, TCGETA, &savtty );
}

restoretty ()

{
	ioctl ( 0, TCSETAW, &savtty );
}

/*
** flush terminal
*/

flushtty ()

{
	ioctl ( 0, TCFLSH, 0 );		/* discard input queue */
}


/*
** get login name
**
** terminate if error or end-of-file
** return 0 if BREAK detected
** return 1 if input accepted
*/

getname ( name, size, tim )
char *name;
int size;
int tim;

{
	register char *np;
	register c;
	char cs;

	signal ( SIGALRM, timedout );
	alarm ( tim );
	prompt ();
	np = name;
	for ( ;; ) {
		if ( read ( 0, &cs, 1 ) <= 0 ) {
			leave ( L_DISC );		/* error */
		}
		if ( ( c = cs & 0177 ) == 0 ) {
			return ( 0 );			/* BREAK */
		}
		if ( c == ttydef.tty.c_cc[VEOF] ) {
			leave ( L_DISC );		/* EOF   */
		}
		if ( c == ttydef.tty.c_cc[VERASE] || c == '#' ) {
			if ( np > name ) {
				np--;
				printf ( "\b \b" );
			}
			continue;
		} else {
			if ( c == ttydef.tty.c_cc[VKILL] || c == '@' ) {
				printf ( "\r" );
				if ( np > name ) {
					prompt (), printf ("          \r" );
				}
				prompt ();
				np = name;
				continue;
			}
		}
		if ( c == '\r' || c == '\n' ) {
			printf ( "\r\n" );
			break;
		}
		if ( np < &name[size] ) {
			*np++ = c;
			printf ( "%c", cs );
		}
	}
	alarm ( 0 );
	signal ( SIGALRM, SIG_DFL );
	*np = 0;
	return ( 1 );
}

prompt ()

{
	flushtty ();
	if ( getty ) {
		printf ( ttydef.login2 );
	} else {
		printf ( ttydef.login0 );
	}
}

/*
** the login timed out
*/

timedout ()

{
	printf ( msgtime );
	leave ( L_BAD );
}


int stopmotd;

/*
** make it possible to end motd display
*/

catch ()

{
	signal ( SIGINT, SIG_IGN );
	stopmotd = 1;
}

/*
** display message of the day
*/

showmotd ()

{
	register c;

	stopmotd = 0;
	signal ( SIGINT, catch );
	if ( ( file = fopen ( Fmotd, "r" ) ) != NULL ) {
		while ( ( ( c = getc ( file ) ) != EOF ) && ( stopmotd == 0 ) ) {
			putchar ( c );
		}
		fclose ( file );
		fflush ( stdout );
	}
	signal ( SIGINT, SIG_IGN );
}


/*
** Set the value of var to be value in the Unix environment env.
** Var should end with '='.
** (bindings are of the form "var=value")
** This procedure assumes the memory for the first level of environ
** was allocated using malloc.
*/
 
setenv ( var, value, clobber )
char *var;
char *value;
int clobber;

{
	extern char **environ;
	int ix;
	int varlen = strlen ( var );
	int vallen = strlen ( value );

	for ( ix = 0; environ[ix] != NULL; ix++ ) {
		if ( strncmp ( environ[ix], var, varlen ) == 0) {
			/* found it */
			if ( !clobber ) {
				return;
			}
			environ[ix] = malloc ( varlen + vallen + 1 );
			strcpy ( environ[ix], var );
			strcat ( environ[ix], value );
			return;
		}
	}
	environ = (char **) realloc ( environ, sizeof (char *) * (ix + 2) );
	if ( environ == NULL ) {
		printf ( msgmalloc );
		leave ( L_BAD );
	}
	environ[ix] = malloc ( varlen + vallen + 1 );
	strcpy ( environ[ix], var );
	strcat ( environ[ix], value );
	environ[++ix] = NULL;
}


/*
** get the group id of the guy who owns all terminals
*/

tty_gid ( default_gid )
int default_gid;

{
	struct group *getgrnam(), *gr;
	int gid = default_gid;

	gr = getgrnam ( "tty" );
	if ( gr != (struct group *) 0 ) {
		gid = gr->gr_gid;
	}
	endgrent ();

	return ( gid );
}

closeacc(username)
char	username[32] ;
{
	FILE	*io ;
	long	place ;
	long	max ;
	int	i,x,z ; 
	char	line[256] ; 
	char	flag = 0 ; 
			
	if ((io = fopen(Fpassd,"r+")) == NULL) { return(-1) ; }
	
	fseek(io,0L,2) ; 
	max = ftell(io) ; 
	fseek(io,0L,0) ;
	
	while(place < max)
	{
		i = 0 ; 
		place = ftell(io) ; 
		fgets(line,256,io) ; 
		if (instr(0,username,line) == 0)
		{ flag = 1 ; break ; }
	}			
	if (flag == 0) { return(0) ; }
	while(++i < 256 && line[i+1] != ':') ;
	x = i+=2 ;
	while(++x < 256 && line[x+1] != ':') ;
	if (line[i] == ':' || line[i+1] == ':') { return(0) ; }
	fseek(io,place+i,0) ; 
	x++ ;
	for (z = i ; z < x ; z++) { fputc('*',io) ; }
	fclose(io) ; 
	if ((io = fopen("/dev/bclock","r")) == NULL) { strcpy(line,"time?") ; }
	else
	{
		fgets(line,256,io) ; 
		fclose(io) ;
	}
	if ((io = fopen("/usr/adm/closed","a")) == NULL) { return(-1) ; }
	fprintf(io,"%s closed logerrs %s\n",username) ; 
	fclose(io) ; 	
}

/*************************************************************
** Command:
**	0	Display actual age
**	1	Age password one day
**	2	Return days remaining
**	3	Enter new age data 
**	4	Zero days.
*/

checkname(name,command)
char	name[256] ; 
int	command ;
{
	FILE	*io ; 
	long	place,
		spot,
		max ; 
	int	i,z,x ;
	int	age ; 
	int	days ; 
	int	c,d ;
	char	b ; 
	char	line[256] ; 
					
	if ((io = fopen("passwd","r+")) == NULL) { return(-1) ; }
	
	fseek(io,0L,2) ; 
	max = ftell(io) ; 
	fseek(io,0L,0) ; 

	while(1)
	{
		z = i = 0 ; 
		while(b != ':') { b = fgetc(io) ; line[i++] = b ; }
		if (instr(0,name,line) == 0) { break ; }
		while(b != '\n') { b = fgetc(io) ; }
		if (ftell(io) >= max) { fclose(io) ; return(0) ; } 
	}

	while(z < 6)
	{
		b = fgetc(io) ; 
		if (b == ':') { z++ ; }
		if (ftell(io) > max) { return(0) ; }
	}
	spot = ftell(io) ; 
	age = (fgetc(io) -'A')*10 ;	age+= fgetc(io)-'A' ;
	b = fgetc(io) ; 
	days = (fgetc(io) -'A')*10 ;	days+= fgetc(io)-'A' ;
	
	switch(command)
	{
		case	2:	fclose(io) ; 
				if ((age-days) < 0)
				{
					printf("Your password has expired!\n") ; 
					printf("Please use 'passwd' to change.\n") ; 
					return(-1) ; 
				}
				return(0) ; 
		case	4:	fseek(io,spot+3L,0) ; 
				fputc('A',io) ; 
				fputc('A',io) ;
				break ; 
	}
	fclose(io) ; 
	return(0) ; 
}		

